@@ -1,7 +1,7 @@
 /* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* vim: set ts=2 et sw=2 tw=80: */
 /*
-//@line 46 "/builds/slave/m-beta-linuxqt-xr/build/toolkit/mozapps/downloads/nsHelperAppDlg.js"
+//@line 46 "/builds/slave/m-beta-lnx-xr/build/toolkit/mozapps/downloads/nsHelperAppDlg.js"
 */
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -268,9 +268,13 @@ nsUnknownContentTypeDialog.prototype = {
                             .getService(Components.interfaces.nsIDownloadManager);
     picker.displayDirectory = dnldMgr.userDownloadsDirectory;
 
+    var relatedURI = null;
+    if (aContext.document)
+      relatedURI = aContext.document.documentURIObject;
+
     // The last directory preference may not exist, which will throw.
     try {
-      var lastDir = gDownloadLastDir.file;
+      var lastDir = gDownloadLastDir.getFile(relatedURI);
       if (isUsableDirectory(lastDir))
         picker.displayDirectory = lastDir;
     }
@@ -299,7 +303,7 @@ nsUnknownContentTypeDialog.prototype = {
       var newDir = result.parent.QueryInterface(Components.interfaces.nsILocalFile);
 
       // Do not store the last save directory as a pref inside the private browsing mode
-      gDownloadLastDir.file = newDir;
+      gDownloadLastDir.setFile(relatedURI, newDir);
 
       result = this.validateLeafName(newDir, result.leafName, null);
     }
@@ -337,7 +341,7 @@ nsUnknownContentTypeDialog.prototype = {
 
     var createdFile = DownloadPaths.createNiceUniqueFile(aLocalFile);
 
-//@line 397 "/builds/slave/m-beta-linuxqt-xr/build/toolkit/mozapps/downloads/nsHelperAppDlg.js"
+//@line 401 "/builds/slave/m-beta-lnx-xr/build/toolkit/mozapps/downloads/nsHelperAppDlg.js"
 
     return createdFile;
   },
@@ -411,7 +415,7 @@ nsUnknownContentTypeDialog.prototype = {
       // want users to be able to autodownload .exe files.
       var rememberChoice = this.dialogElement("rememberChoice");
 
-//@line 491 "/builds/slave/m-beta-linuxqt-xr/build/toolkit/mozapps/downloads/nsHelperAppDlg.js"
+//@line 495 "/builds/slave/m-beta-lnx-xr/build/toolkit/mozapps/downloads/nsHelperAppDlg.js"
       if (shouldntRememberChoice) {
         rememberChoice.checked = false;
         rememberChoice.disabled = true;
@@ -551,12 +555,12 @@ nsUnknownContentTypeDialog.prototype = {
   // Returns true if opening the default application makes sense.
   openWithDefaultOK: function() {
     // The checking is different on Windows...
-//@line 641 "/builds/slave/m-beta-linuxqt-xr/build/toolkit/mozapps/downloads/nsHelperAppDlg.js"
+//@line 645 "/builds/slave/m-beta-lnx-xr/build/toolkit/mozapps/downloads/nsHelperAppDlg.js"
     // On other platforms, default is Ok if there is a default app.
     // Note that nsIMIMEInfo providers need to ensure that this holds true
     // on each platform.
     return this.mLauncher.MIMEInfo.hasDefaultHandler;
-//@line 646 "/builds/slave/m-beta-linuxqt-xr/build/toolkit/mozapps/downloads/nsHelperAppDlg.js"
+//@line 650 "/builds/slave/m-beta-lnx-xr/build/toolkit/mozapps/downloads/nsHelperAppDlg.js"
   },
 
   // Set "default" application description field.
@@ -577,9 +581,9 @@ nsUnknownContentTypeDialog.prototype = {
 
   // getPath:
   getPath: function (aFile) {
-//@line 669 "/builds/slave/m-beta-linuxqt-xr/build/toolkit/mozapps/downloads/nsHelperAppDlg.js"
+//@line 673 "/builds/slave/m-beta-lnx-xr/build/toolkit/mozapps/downloads/nsHelperAppDlg.js"
     return aFile.path;
-//@line 671 "/builds/slave/m-beta-linuxqt-xr/build/toolkit/mozapps/downloads/nsHelperAppDlg.js"
+//@line 675 "/builds/slave/m-beta-lnx-xr/build/toolkit/mozapps/downloads/nsHelperAppDlg.js"
   },
 
   // initAppAndSaveToDiskValues:
@@ -809,7 +813,7 @@ nsUnknownContentTypeDialog.prototype = {
         // for the file to be saved to to pass to |saveToDisk| - otherwise
         // we must ask the user to pick a save name.
 
-//@line 914 "/builds/slave/m-beta-linuxqt-xr/build/toolkit/mozapps/downloads/nsHelperAppDlg.js"
+//@line 918 "/builds/slave/m-beta-lnx-xr/build/toolkit/mozapps/downloads/nsHelperAppDlg.js"
 
         // see @notify
         // we cannot use opener's setTimeout, see bug 420405
@@ -864,13 +868,13 @@ nsUnknownContentTypeDialog.prototype = {
   // Retrieve the pretty description from the file
   getFileDisplayName: function getFileDisplayName(file)
   {
-//@line 982 "/builds/slave/m-beta-linuxqt-xr/build/toolkit/mozapps/downloads/nsHelperAppDlg.js"
+//@line 986 "/builds/slave/m-beta-lnx-xr/build/toolkit/mozapps/downloads/nsHelperAppDlg.js"
     return file.leafName;
   },
 
   // chooseApp:  Open file picker and prompt user for application.
   chooseApp: function() {
-//@line 1030 "/builds/slave/m-beta-linuxqt-xr/build/toolkit/mozapps/downloads/nsHelperAppDlg.js"
+//@line 1034 "/builds/slave/m-beta-lnx-xr/build/toolkit/mozapps/downloads/nsHelperAppDlg.js"
     var nsIFilePicker = Components.interfaces.nsIFilePicker;
     var fp = Components.classes["@mozilla.org/filepicker;1"]
                        .createInstance(nsIFilePicker);
@@ -887,7 +891,7 @@ nsUnknownContentTypeDialog.prototype = {
                    createInstance(Components.interfaces.nsILocalHandlerApp);
       localHandlerApp.executable = fp.file;
       this.chosenApp = localHandlerApp;
-//@line 1047 "/builds/slave/m-beta-linuxqt-xr/build/toolkit/mozapps/downloads/nsHelperAppDlg.js"
+//@line 1051 "/builds/slave/m-beta-lnx-xr/build/toolkit/mozapps/downloads/nsHelperAppDlg.js"
 
       // Show the "handler" menulist since we have a (user-specified)
       // application now.
